#include "All_old.h"
#include "Prepare_old.h"

#define BIT(x)  ((unsigned int)(x) >= 32u ? 0 : 1lu << (x))
#define X(b)    BIT(32-1-(b))
#define P32     (X(32)+X(26)+X(23)+X(22)+X(16)+X(12)+X(11)+X(10)+X(8)+X(7)+X(5)+X(4)+X(2)+X(1)+X(0))


static unsigned __int32  CRC32_Table [256];


CPrepare ::
CPrepare ( void )
{
    unsigned __int32  poly = P32;
    unsigned __int32  i;
    unsigned __int32  j;
    unsigned __int32  t = poly;

    memset ( CRC32_Table, 0, sizeof CRC32_Table );

    j = 0x80;
    do {
        i = 0x100;
        do {
            if (--i & j)
                CRC32_Table [i] ^= t;
        } while (i);
        t = (t >> 1) ^ (t & 1 ? poly : 0);
    } while (j >>= 1);
}


#if 1

/*
 * Read an octet from RawData, add contents to 'var' and update CRC
 *   version with 0: assign (=) instead of OR (|=)
 *   version with S: value is treated as signed value (most significantly octet for 16/24/32 bit)
 */

#define READ_BUFFER(var,shift)                                   \
    var |= (unsigned __int8)*pRawData << (shift);                \
    CRC  = ((unsigned __int32)CRC >> 8) ^ CRC32_Table [(CRC & 0xFF) ^ (unsigned __int8)*pRawData++]

#define READ_BUFFER0(var,shift)                                  \
    var  = (unsigned __int8)*pRawData << (shift);                \
    CRC  = ((unsigned __int32)CRC >> 8) ^ CRC32_Table [(CRC & 0xFF) ^ (unsigned __int8)*pRawData++]

#define READ_BUFFERS(var,shift)                                  \
    var |= (signed __int8)(*pRawData) << (shift);                \
    CRC  = ((unsigned __int32)CRC >> 8) ^ CRC32_Table [(CRC & 0xFF) ^ (unsigned __int8)*pRawData++]

#define READ_BUFFERS0(var,shift)                                 \
    var  = (signed __int8)(*pRawData) << (shift);                \
    CRC  = ((unsigned __int32)CRC >> 8) ^ CRC32_Table [(CRC & 0xFF) ^ (unsigned __int8)*pRawData++]


/*
 *  Only special Stereo handling for "pseudo stereo files" with 16 bit ???
 *  Peak handling for 8/24 bit different from 16 bit ???
 *  Unprepare should also do range checking for 8 and 24 bit, not only for 16 bit (throw).
 *
 *  Matrixing is done via: S = L - R, M = R + (S / 2);
 */

int CPrepare ::
Prepare ( unsigned char*      pRawData,         // __int8
          int                 nBytes,           // size_t
          const WAVEFORMATEX* pWaveFormatEx,
          int*                pOutputX,         // __int32
          int*                pOutputY,         // __int32
          unsigned int*       pCRC,             // __int32
          int*                pSpecialCodes,
          int*                pPeakLevel )      // __int32
{
    if ( pRawData == NULL || pWaveFormatEx == NULL )            // error check the parameters
        return ERROR_BAD_PARAMETER;


    *pCRC          = ~0;                                        // initialize the pointers that got passed in
    *pSpecialCodes = 0;

    // variables
    unsigned __int32  CRC            = ~0;
    const int         nBytesPerBlock = pWaveFormatEx->nChannels * ((pWaveFormatEx->wBitsPerSample + 7) >> 3);
    const int         nTotalBlocks   = nBytes / nBytesPerBlock;
    __int32           R;
    __int32           L;
    __int32           RPeak = 0;
    __int32           LPeak = 0;
    int               i;

    // the prepare code

    switch ( pWaveFormatEx->nChannels ) {
    case 2:
        switch ( pWaveFormatEx->wBitsPerSample ) {
        case 16:
            for (i = 0; i < nTotalBlocks; i++) {
                READ_BUFFER0 ( R,  0 );
                READ_BUFFERS ( R,  8 );
                READ_BUFFER0 ( L,  0 );
                READ_BUFFERS ( L,  8 );

                if (abs(L) > LPeak) LPeak = abs(L);             // check the peak
                if (abs(R) > RPeak) RPeak = abs(R);

                pOutputX[i] = R + ((pOutputY[i] = L - R) / 2);  // convert to x,y
            }

            if (LPeak == 0) *pSpecialCodes |= SPECIAL_FRAME_LEFT_SILENCE;
            if (RPeak == 0) *pSpecialCodes |= SPECIAL_FRAME_RIGHT_SILENCE;
            if (LPeak > *pPeakLevel) *pPeakLevel = LPeak;
            if (RPeak > *pPeakLevel) *pPeakLevel = RPeak;

            for ( i = 0; pOutputY[i++] == 0; ) {                // check for pseudo-stereo files
                if ( i == nTotalBlocks ) {
                    *pSpecialCodes |= SPECIAL_FRAME_PSEUDO_STEREO;
                    break;
                }
            }
            break;

        case  8:
            for (i = 0; i < nTotalBlocks; i++) {
                READ_BUFFER0 ( R,  0 );
                READ_BUFFER0 ( L,  0 );
                R -= 128;
                L -= 128;

                if (abs(L) > LPeak) LPeak = abs(L);             // check the peak
                if (abs(R) > RPeak) RPeak = abs(R);

                pOutputX[i] = R + ((pOutputY[i] = L - R) / 2);  // convert to x,y
            }
            if (LPeak > *pPeakLevel) *pPeakLevel = LPeak;
            if (RPeak > *pPeakLevel) *pPeakLevel = RPeak;
            break;

        case 24:
            for (i = 0; i < nTotalBlocks; i++) {
                READ_BUFFER0 ( R,  0 );
                READ_BUFFER  ( R,  8 );
                READ_BUFFERS ( R, 16 );
                READ_BUFFER0 ( L,  0 );
                READ_BUFFER  ( L,  8 );
                READ_BUFFERS ( L, 16 );

                if (abs(L) > LPeak) LPeak = abs(L);             // check the peak
                if (abs(R) > RPeak) RPeak = abs(R);

                pOutputX[i] = R + ((pOutputY[i] = L - R) / 2);  // convert to x,y
            }
            if (LPeak > *pPeakLevel) *pPeakLevel = LPeak;
            if (RPeak > *pPeakLevel) *pPeakLevel = RPeak;
            break;
        }
        break;

    case 1:
        switch ( pWaveFormatEx->wBitsPerSample ) {
        case 16:
            for (i = 0; i < nTotalBlocks; i++) {
                READ_BUFFER0 ( R,  0 );
                READ_BUFFERS ( R,  8 );

                if (abs(R) > RPeak) RPeak = abs(R);             // check the peak

                pOutputX[i] = R;                                // convert to x,y ???
            }

            if ( RPeak > *pPeakLevel ) *pPeakLevel = RPeak;
            if ( RPeak == 0 ) *pSpecialCodes |= SPECIAL_FRAME_MONO_SILENCE;
            break;

        case  8:
            for (i = 0; i < nTotalBlocks; i++) {
                READ_BUFFER0 ( R,  0 );
                R -= 128;

                if (abs(R) > RPeak) RPeak = abs(R);             // check the peak

                pOutputX[i] = R;
            }
            if ( RPeak > *pPeakLevel ) *pPeakLevel = RPeak;
            break;

        case 24:
            for (i = 0; i < nTotalBlocks; i++) {
                READ_BUFFER0 ( R,  0 );
                READ_BUFFER  ( R,  8 );
                READ_BUFFERS ( R, 16 );

                if (abs(R) > RPeak) RPeak = abs(R);             // check the peak

                pOutputX[i] = R;
            }
            if ( RPeak > *pPeakLevel ) *pPeakLevel = RPeak;
            break;
        }
        break;
    }

    CRC = ~CRC >> 1;                                            // add the special code

    if ( *pSpecialCodes != 0 )
        CRC |= 1L << 31;

    *pCRC = CRC;
    return 0;
}

#undef READ_BUFFER
#undef READ_BUFFER0
#undef READ_BUFFERS
#undef READ_BUFFERS0


/*
 *  Function don't warn if pWaveFormatEx->nChannels or pWaveFormatEx->wBitsPerSample are
 *  illegal. DANGER OF DATA LOSS !!! Note that pWaveFormatEx->wBitsPerSample==12
 *  and samples in the form xxxxxxxxxxxx0000 is a valid WAV file!
 */

/*
 * The following macro writes an octet to the Output buffer and updates CRC
 */

#define WRITE_BUFFER(byte)  \
    *pCRC = (*pCRC >> 8) ^ CRC32_Table [(*pCRC & 0xFF) ^ (*pOutput++ = (unsigned __int8)(byte))]

/*
 *  May be this function also should be block based. This improves performance
 *  Block sizes of 44...441 samples per channel would be enough. Note that sample wise decoding has no advantages,
 *  you still need to block the data for further processing. You can write 32 bit to a file. You can't write
 *  32 bit directly to a sound card.
 *  It is not nice to fall from one extreme (huge blocks of >> 10000 samples) to another (1 sample).
 */

void CPrepare ::
Unprepare ( int                  X,              // __int32
            int                  Y,              // __int32
            const WAVEFORMATEX*  pWaveFormatEx,
            unsigned char*       pOutput,        // __int8
            unsigned int*        pCRC )          // __int32
{
    __int32  nR;
    __int32  nL;

    // decompress and convert from (x,y) -> (l,r)
    switch ( pWaveFormatEx->nChannels ) {
    case 2:
        switch ( pWaveFormatEx->wBitsPerSample ) {
        case 16:
            nR = X - (Y / 2);                                   // get the right and left values
            nL = nR + Y;

            if ( nR != (__int16)nR  ||  nL != (__int16)nL )     // error check (for overflows)
                throw (-2);

            WRITE_BUFFER (nR >>  0);
            WRITE_BUFFER (nR >>  8);
            WRITE_BUFFER (nL >>  0);
            WRITE_BUFFER (nL >>  8);
            break;

        case  8:
            nR = X - (Y / 2) + 128;
            nL = nR + Y;
            WRITE_BUFFER (nR);
            WRITE_BUFFER (nL);
            break;

        case 24:
            nR = X - (Y / 2);
            nL = nR + Y;
            WRITE_BUFFER (nR >>  0);
            WRITE_BUFFER (nR >>  8);
            WRITE_BUFFER (nR >> 16);
            WRITE_BUFFER (nL >>  0);
            WRITE_BUFFER (nL >>  8);
            WRITE_BUFFER (nL >> 16);
            break;
        }
        break;

    case 1:
        switch ( pWaveFormatEx->wBitsPerSample ) {
        case 16:
            WRITE_BUFFER (X >>  0);
            WRITE_BUFFER (X >>  8);
            break;

        case  8:
            WRITE_BUFFER (X + 128);
            break;

        case 24:
            WRITE_BUFFER (X >>  0);
            WRITE_BUFFER (X >>  8);
            WRITE_BUFFER (X >> 16);
            break;
        }
    }
}

#else

//
// What do this code exactly do ?????
//

int CPrepare::Prepare(unsigned char * pRawData, int nBytes, const WAVEFORMATEX * pWaveFormatEx, int * pOutputX, int *pOutputY, unsigned int *pCRC, int *pSpecialCodes, int *pPeakLevel)
{
    // error check the parameters
    if (pRawData == NULL || pWaveFormatEx == NULL)
        return ERROR_BAD_PARAMETER;

    // initialize the pointers that got passed in
    *pCRC = 0xFFFFFFFF;
    *pSpecialCodes = 0;

    // variables
    unsigned __int32 CRC = 0xFFFFFFFF;
    const int nTotalBlocks = nBytes / pWaveFormatEx->nBlockAlign;
    int R,L;

    // the prepare code

    if (pWaveFormatEx->wBitsPerSample == 8)
    {
        if (pWaveFormatEx->nChannels == 2)
        {
            for (int nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
            {
                R = (int) (*((unsigned char *) pRawData) - 128);
                L = (int) (*((unsigned char *) (pRawData + 1)) - 128);

                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                // check the peak
                if (labs(L) > *pPeakLevel)
                    *pPeakLevel = labs(L);
                if (labs(R) > *pPeakLevel)
                    *pPeakLevel = labs(R);

                // convert to x,y
                pOutputY[nBlockIndex] = L - R;
                pOutputX[nBlockIndex] = R + (pOutputY[nBlockIndex] / 2);
            }
        }
        else if (pWaveFormatEx->nChannels == 1)
        {
            for (int nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
            {
                R = (int) (*((unsigned char *) pRawData) - 128);

                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                // check the peak
                if (labs(R) > *pPeakLevel)
                    *pPeakLevel = labs(R);

                // convert to x,y
                pOutputX[nBlockIndex] = R;
            }
        }
    }
    else if (pWaveFormatEx->wBitsPerSample == 24)
    {
        if (pWaveFormatEx->nChannels == 2)
        {
            for (int nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
            {
                unsigned __int32 nTemp = 0;

                nTemp |= (*pRawData << 0);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                nTemp |= (*pRawData << 8);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                nTemp |= (*pRawData << 16);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                if (nTemp & 0x800000)
                    R = (int) (nTemp & 0x7fffff) - 0x800000;
                else
                    R = (int) (nTemp & 0x7fffff);

                nTemp = 0;

                nTemp |= (*pRawData << 0);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                nTemp |= (*pRawData << 8);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                nTemp |= (*pRawData << 16);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                if (nTemp & 0x800000)
                    L = (int) (nTemp & 0x7fffff) - 0x800000;
                else
                    L = (int) (nTemp & 0x7fffff);

                // check the peak
                if (labs(L) > *pPeakLevel)
                    *pPeakLevel = labs(L);
                if (labs(R) > *pPeakLevel)
                    *pPeakLevel = labs(R);

                // convert to x,y
                pOutputY[nBlockIndex] = L - R;
                pOutputX[nBlockIndex] = R + (pOutputY[nBlockIndex] / 2);

            }
        }
        else if (pWaveFormatEx->nChannels == 1)
        {
            for (int nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
            {
                unsigned __int32 nTemp = 0;

                nTemp |= (*pRawData << 0);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                nTemp |= (*pRawData << 8);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                nTemp |= (*pRawData << 16);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                if (nTemp & 0x800000)
                    R = (int) (nTemp & 0x7fffff) - 0x800000;
                else
                    R = (int) (nTemp & 0x7fffff);

                // check the peak
                if (labs(R) > *pPeakLevel)
                    *pPeakLevel = labs(R);

                // convert to x,y
                pOutputX[nBlockIndex] = R;
            }
        }
    }
    else
    {
        if (pWaveFormatEx->nChannels == 2)
        {
            int LPeak = 0;
            int RPeak = 0;
            int nBlockIndex = 0;
            for (nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
            {

                R = (int) *((__int16 *) pRawData);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                L = (int) *((__int16 *) pRawData);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                // check the peak
                if (labs(L) > LPeak)
                    LPeak = labs(L);
                if (labs(R) > RPeak)
                    RPeak = labs(R);

                // convert to x,y
                pOutputY[nBlockIndex] = L - R;
                pOutputX[nBlockIndex] = R + (pOutputY[nBlockIndex] / 2);
            }

            if (LPeak == 0) { *pSpecialCodes |= SPECIAL_FRAME_LEFT_SILENCE; }
            if (RPeak == 0) { *pSpecialCodes |= SPECIAL_FRAME_RIGHT_SILENCE; }
            if (max(LPeak, RPeak) > *pPeakLevel)
            {
                *pPeakLevel = max(LPeak, RPeak);
            }

            // check for pseudo-stereo files
            nBlockIndex = 0;
            while (pOutputY[nBlockIndex++] == 0)
            {
                if (nBlockIndex == (nBytes / 4))
                {
                    *pSpecialCodes |= SPECIAL_FRAME_PSEUDO_STEREO;
                    break;
                }
            }

        }
        else if (pWaveFormatEx->nChannels == 1)
        {
            int nPeak = 0;
            for (int nBlockIndex = 0; nBlockIndex < nTotalBlocks; nBlockIndex++)
            {
                R = (int) *((__int16 *) pRawData);

                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *pRawData++];

                // check the peak
                if (labs(R) > nPeak)
                    nPeak = labs(R);

                //convert to x,y
                pOutputX[nBlockIndex] = R;
            }

            if (nPeak > *pPeakLevel)
                *pPeakLevel = nPeak;
            if (nPeak == 0) { *pSpecialCodes |= SPECIAL_FRAME_MONO_SILENCE; }
        }
    }

    CRC = CRC ^ 0xFFFFFFFF;

    // add the special code
    CRC >>= 1;

    if (*pSpecialCodes != 0)
    {
        CRC |= (1 << 31);
    }

    *pCRC = CRC;

    return 0;
}

void CPrepare::Unprepare(int X, int Y, const WAVEFORMATEX * pWaveFormatEx, unsigned char * pOutput, unsigned int * pCRC)
{
    #define CALCULATE_CRC_BYTE  *pCRC = (*pCRC >> 8) ^ CRC32_Table[(*pCRC & 0xFF) ^ *pOutput++];
    // decompress and convert from (x,y) -> (l,r)
    // sort of long and ugly.... sorry

    if (pWaveFormatEx->nChannels == 2)
    {
        if (pWaveFormatEx->wBitsPerSample == 16)
        {
            // get the right and left values
            int nR = X - (Y / 2);
            int nL = nR + Y;

            // error check (for overflows)
            if ((nR < -32768) || (nR > 32767) || (nL < -32768) || (nL > 32767))
            {
                throw(-2);
            }

            *(__int16 *) pOutput = (__int16) nR;
            CALCULATE_CRC_BYTE
            CALCULATE_CRC_BYTE

            *(__int16 *) pOutput = (__int16) nL;
            CALCULATE_CRC_BYTE
            CALCULATE_CRC_BYTE
        }
        else if (pWaveFormatEx->wBitsPerSample == 8)
        {
            unsigned char R = (X - (Y / 2) + 128);
            *pOutput = R;
            CALCULATE_CRC_BYTE
            *pOutput = (unsigned char) (R + Y);
            CALCULATE_CRC_BYTE
        }
        else if (pWaveFormatEx->wBitsPerSample == 24)
        {
            __int32 RV, LV;

            RV = X - (Y / 2);
            LV = RV + Y;

            unsigned __int32 nTemp = 0;
            if (RV < 0)
                nTemp = ((unsigned __int32) (RV + 0x800000)) | 0x800000;
            else
                nTemp = (unsigned __int32) RV;

            *pOutput = (unsigned char) ((nTemp >> 0) & 0xFF);
            CALCULATE_CRC_BYTE
            *pOutput = (unsigned char) ((nTemp >> 8) & 0xFF);
            CALCULATE_CRC_BYTE
            *pOutput = (unsigned char) ((nTemp >> 16) & 0xFF);
            CALCULATE_CRC_BYTE

            nTemp = 0;
            if (LV < 0)
                nTemp = ((unsigned __int32) (LV + 0x800000)) | 0x800000;
            else
                nTemp = (unsigned __int32) LV;

            *pOutput = (unsigned char) ((nTemp >> 0) & 0xFF);
            CALCULATE_CRC_BYTE

            *pOutput = (unsigned char) ((nTemp >> 8) & 0xFF);
            CALCULATE_CRC_BYTE

            *pOutput = (unsigned char) ((nTemp >> 16) & 0xFF);
            CALCULATE_CRC_BYTE
        }
    }
    else if (pWaveFormatEx->nChannels == 1)
    {
        if (pWaveFormatEx->wBitsPerSample == 16)
        {
            __int16 R = X;

            *(__int16 *) pOutput = (__int16) R;
            CALCULATE_CRC_BYTE
            CALCULATE_CRC_BYTE
        }
        else if (pWaveFormatEx->wBitsPerSample == 8)
        {
            unsigned char R = X + 128;
            *pOutput = R;
            CALCULATE_CRC_BYTE
        }
        else if (pWaveFormatEx->wBitsPerSample == 24)
        {
            __int32 RV = X;

            unsigned __int32 nTemp = 0;
            if (RV < 0)
                nTemp = ((unsigned __int32) (RV + 0x800000)) | 0x800000;
            else
                nTemp = (unsigned __int32) RV;

            *pOutput = (unsigned char) ((nTemp >> 0) & 0xFF);
            CALCULATE_CRC_BYTE
            *pOutput = (unsigned char) ((nTemp >> 8) & 0xFF);
            CALCULATE_CRC_BYTE
            *pOutput = (unsigned char) ((nTemp >> 16) & 0xFF);
            CALCULATE_CRC_BYTE
        }
    }
}


#endif




///////////////////////////////////////////////////////////////////////////////////////////////////




#ifdef BACKWARDS_COMPATIBILITY

int CPrepare::UnprepareOld(int *pInputX, int *pInputY, int nBlocks, const WAVEFORMATEX *pWaveFormatEx, unsigned char *pRawData, unsigned int *pCRC, int *pSpecialCodes, int nFileVersion)
{
    //the CRC that will be figured during decompression
    unsigned __int32 CRC = 0xffffffff;

    //decompress and convert from (x,y) -> (l,r)
    //sort of int and ugly.... sorry
    if (pWaveFormatEx->nChannels == 2)
    {
        //convert the x,y data to raw data
        if (pWaveFormatEx->wBitsPerSample == 16)
        {
            __int16 R;
            unsigned char *Buffer = &pRawData[0];
            int *pX = pInputX;
            int *pY = pInputY;

            for (; pX < &pInputX[nBlocks]; pX++, pY++)
            {
                R = *pX - (*pY / 2);

                *(__int16 *) Buffer = (__int16) R;
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                *(__int16 *) Buffer = (__int16) R + *pY;
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];
            }
        }
        else if (pWaveFormatEx->wBitsPerSample == 8)
        {
            unsigned char *R = (unsigned char *) &pRawData[0];
            unsigned char *L = (unsigned char *) &pRawData[1];

            if (nFileVersion > 3830)
            {
                for (int SampleIndex = 0; SampleIndex < nBlocks; SampleIndex++, L+=2, R+=2)
                {
                    *R = (unsigned char) (pInputX[SampleIndex] - (pInputY[SampleIndex] / 2) + 128);
                    CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *R];
                    *L = (unsigned char) (*R + pInputY[SampleIndex]);
                    CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *L];
                }
            }
            else
            {
                for (int SampleIndex = 0; SampleIndex < nBlocks; SampleIndex++, L+=2, R+=2)
                {
                    *R = (unsigned char) (pInputX[SampleIndex] - (pInputY[SampleIndex] / 2));
                    CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *R];
                    *L = (unsigned char) (*R + pInputY[SampleIndex]);
                    CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *L];

                }
            }
        }
        else if (pWaveFormatEx->wBitsPerSample == 24)
        {
            unsigned char *Buffer = (unsigned char *) &pRawData[0];
            __int32 RV, LV;

            for (int SampleIndex = 0; SampleIndex < nBlocks; SampleIndex++)
            {
                RV = pInputX[SampleIndex] - (pInputY[SampleIndex] / 2);
                LV = RV + pInputY[SampleIndex];

                unsigned __int32 nTemp = 0;
                if (RV < 0)
                    nTemp = ((unsigned __int32) (RV + 0x800000)) | 0x800000;
                else
                    nTemp = (unsigned __int32) RV;

                *Buffer = (unsigned char) ((nTemp >> 0) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                *Buffer = (unsigned char) ((nTemp >> 8) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                *Buffer = (unsigned char) ((nTemp >> 16) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                nTemp = 0;
                if (LV < 0)
                    nTemp = ((unsigned __int32) (LV + 0x800000)) | 0x800000;
                else
                    nTemp = (unsigned __int32) LV;

                *Buffer = (unsigned char) ((nTemp >> 0) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                *Buffer = (unsigned char) ((nTemp >> 8) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                *Buffer = (unsigned char) ((nTemp >> 16) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];
            }
        }
    }
    else if (pWaveFormatEx->nChannels == 1)
    {
        //convert to raw data
        if (pWaveFormatEx->wBitsPerSample == 8)
        {
            unsigned char *R = (unsigned char *) &pRawData[0];

            if (nFileVersion > 3830)
            {
                for (int SampleIndex = 0; SampleIndex < nBlocks; SampleIndex++, R++)
                {
                    *R = pInputX[SampleIndex] + 128;
                    CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *R];
                }
            }
            else
            {
                for (int SampleIndex = 0; SampleIndex < nBlocks; SampleIndex++, R++)
                {
                    *R = (unsigned char) (pInputX[SampleIndex]);
                    CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *R];
                }
            }

        }
        else if (pWaveFormatEx->wBitsPerSample == 24)
        {

            unsigned char *Buffer = (unsigned char *) &pRawData[0];
            __int32 RV;
            for (int SampleIndex = 0; SampleIndex<nBlocks; SampleIndex++)
            {
                RV = pInputX[SampleIndex];

                unsigned __int32 nTemp = 0;
                if (RV < 0)
                    nTemp = ((unsigned __int32) (RV + 0x800000)) | 0x800000;
                else
                    nTemp = (unsigned __int32) RV;

                *Buffer = (unsigned char) ((nTemp >> 0) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                *Buffer = (unsigned char) ((nTemp >> 8) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];

                *Buffer = (unsigned char) ((nTemp >> 16) & 0xFF);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];
            }
        }
        else
        {
            unsigned char *Buffer = &pRawData[0];

            for (int SampleIndex = 0; SampleIndex < nBlocks; SampleIndex++)
            {
                *(__int16 *) Buffer = (__int16) (pInputX[SampleIndex]);
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];
                CRC = (CRC >> 8) ^ CRC32_Table[(CRC & 0xFF) ^ *Buffer++];
            }
        }
    }

    CRC = CRC ^ 0xffffffff;

    *pCRC = CRC;

    return 0;
}

#endif // BACKWARDS_COMPATIBILITY
